e ) {
							notify_admin_fatal_error( $e );
						}
					}
				);
			}
		}
	}

	/**
	 * Enqueues the entrypoint JavaScript index.js, and declares relevant js
	 * dependencies.
	 *
	 * @param bool $with_icon_chooser
	 * @ignore
	 * @internal
	 * @return string $main_js_handle
	 */
	private function enqueue_admin_js_assets( $with_icon_chooser ) {
		global $wp_version;

		$enable_icon_chooser = boolval( $with_icon_chooser );

		$deps = array();

		/**
		 * If we're on a recent enough version of WordPress 5, then the supporting
		 * libraries are adequate for us to use as externals.
		 *
		 * For earlier versions, we'll need to load our own compatibility bundle,
		 * and disable Gutenberg integration, since our compatibility bundle
		 * uses a newer version of React than what would be available in WordPress
		 * Core in that earlier version.
		 */
		if ( $this->compat_js_required() ) {
			$wp4_compat_resource_handle = self::ADMIN_RESOURCE_HANDLE . '-compat';

			wp_enqueue_script(
				$wp4_compat_resource_handle,
				trailingslashit( FONTAWESOME_DIR_URL ) . 'compat-js/build/compat.js',
				array(),
				self::PLUGIN_VERSION,
				true
			);

			// We need our main bundle to depend on the compat bundle.
			array_push( $deps, $wp4_compat_resource_handle );
		} else {
			$deps = array_merge( $deps, array( 'react', 'react-dom', 'wp-i18n', 'wp-element', 'wp-components', 'wp-api-fetch' ) );

			/**
			 * We don't need these Gutenberg dependencies unless we're on a Gutenberg
			 * page. Declaring them unnecessarily (when not on a Gutenberg page)
			 * has resulted in conflict for at least one other plugin: RankMath.
			 *
			 * See: https://wordpress.org/support/topic/plugin-conflicts-with-rankmath
			 */
			if ( $enable_icon_chooser && $this->is_gutenberg_page() ) {
				$deps = array_merge( $deps, array( 'wp-blocks', 'wp-editor', 'wp-rich-text', 'wp-block-editor' ) );
			}
		}

		if ( $enable_icon_chooser ) {
			/**
			 * TODO: re-enable the case where TinyMCE and Gutenberg are present on the same
			 * page load. For now, we're eliminating that case because
			 * some customers experienced Gutenberg failures on pages where both
			 * editors were active.
			 *
			 * If we're not on a Gutenberg (as plugin) or Block Editor (as WP 5 Core editor),
			 * then we want to enable our TinyMCE integration. We'll initialize it
			 * on the wp_tiny_mce_init action hook.
			 *
			 * According to the docs:
			 * "Fires after tinymce.js is loaded, but before any TinyMCE editor instances are created."
			 *
			 * So we expect this to only fire once, even if multiple instances of the editor
			 * are added to a single page.
			 *
			 * If TinyMCE is not present or not active, then this action hook will
			 * never be fired and thus our TinyMCE integration will never be setup,
			 * which is what we want.
			 */
			add_action( 'wp_tiny_mce_init', array( $this, 'print_classic_editor_icon_chooser_setup_script' ) );
		}

		wp_enqueue_script(
			self::ADMIN_RESOURCE_HANDLE,
			trailingslashit( $this->get_webpack_asset_url_base() ) . 'index.js',
			$deps,
			self::PLUGIN_VERSION,
			true
		);
	}

	/**
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 */
	public function common_data_for_js_bundle() {
		return array(
			'apiNonce'                      => wp_create_nonce( 'wp_rest' ),
			'apiUrl'                        => rest_url( self::REST_API_NAMESPACE ),
			'restApiNamespace'              => self::REST_API_NAMESPACE,
			'rootUrl'                       => rest_url(),
			'detectConflictsUntil'          => $this->detect_conflicts_until(),
			'unregisteredClients'           => $this->unregistered_clients(),
			'showConflictDetectionReporter' => $this->detecting_conflicts(),
			'settingsPageUrl'               => $this->settings_page_url(),
			'activeAdminTab'                => $this->active_admin_tab(),
			'options'                       => $this->options(),
			'webpackPublicPath'             => trailingslashit( FONTAWESOME_DIR_URL ) . 'admin/build/',
			'usingCompatJs'                 => $this->compat_js_required(),
			'isGutenbergPage'               => $this->is_gutenberg_page(),
		);
	}

	/**
	 * Enqueues a kit loader <script>.
	 *
	 * Internal use only, not part of this plugin's public API