CONFLICT_DETECTION_OPTIONS = array(
		'detectConflictsUntil' => 0,
		'unregisteredClients'  => array(),
	);

	/**
	 * @internal
	 * @ignore
	 */
	protected static $instance = null;

	/**
	 * @internal
	 * @ignore
	 */
	protected $client_preferences = array();

	/**
	 * @internal
	 * @ignore
	 */
	protected $icon_chooser_screens = array( 'post.php', 'post-new.php' );

	/**
	 * @internal
	 * @ignore
	 */
	protected $conflicts_by_client = null;

	/**
	 * @internal
	 * @ignore
	 */
	protected $screen_id = null;

	/**
	 * This tracks the state of whether, when we process options after the
	 * plugin upgrades from using the v1 options schema to v2, the former
	 * removeUnregisteredClients option was set. If so we use some automatic
	 * conflict detection and resolution, like that old feature worked.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @deprecated
	 * @internal
	 * @ignore
	 */
	protected $old_remove_unregistered_clients = false;

	/**
	 * Returns the singleton instance of the FontAwesome plugin.
	 *
	 * @since 4.0.0
	 *
	 * @see fa()
	 * @return FontAwesome
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	private function __construct() {
		/* noop */
	}

	/**
	 * Returns this plugin's admin page's screen_id. Only valid after the admin_menu hook has run.
	 *
	 * Internal only, not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 */
	public function admin_screen_id() {
		return $this->screen_id;
	}

	/**
	 * Main entry point for running the plugin. Called automatically when the plugin is loaded.
	 *
	 * Internal only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	public function run() {
		$this->init();

		$this->initialize_rest_api();

		if ( is_admin() ) {
			$this->initialize_admin();
		}
	}

	/**
	 * Callback for init.
	 *
	 * Internal use only.
	 *
	 * @ignore
	 * @internal
	 */
	public function init() {
		try {
			$this->try_upgrade();

			add_shortcode(
				self::SHORTCODE_TAG,
				array( $this, 'process_shortcode' )
			);

			$this->validate_options( fa()->options() );

			try {
				$this->gather_preferences();
			// phpcs:ignore Generic.CodeAnalysis.EmptyStatement.DetectedCatch
			} catch ( PreferenceRegistrationException $e ) {
				/**
				 * Ignore this on normal page loads.
				 * If something seems amiss, the site owner may try to look
				 * into it on the plugin settings page where some additional
				 * diagnostic information may be found.
				 */
			}

			$this->maybe_enqueue_admin_assets();

			// Setup JavaScript internationalization if we're on WordPress 5.0+.
			if ( function_exists( 'wp_set_script_translations' ) ) {
				wp_set_script_translations( self::ADMIN_RESOURCE_HANDLE, 'font-awesome' );
			}

			if ( $this->using_kit() ) {
				$this->enqueue_kit( $this->options()['kitToken'] );
			} else {
				$resource_collection = $this->cdn_resource_collection_for_current_options();
				$this->enqueue_cdn( $this->options(), $resource_collection );
			}
		} catch ( Exception $e ) {
			notify_admin_fatal_error( $e );
		} catch ( Error $e ) {
			notify_admin_fatal_error( $e );
		}
	}

	/**
	 * Not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 * @throws ConfigCorruptionException
	 * @return array
	 */
	public function cdn_resource_collection_for_current_options() {
		return FontAwesome_Release_Provider::get_resource_collection(
			$this->options()['version'],
			array(
				'use_pro'           => $this->pro(),
				'use_svg'           => 'svg' === $this->technology(),
				'use_compatibility' => $this->v4_compatibility(),
			)
		);
	}

	/**
	 * Detects whether upgrade is necessary and performs upgrade if so.
	 *
	 * Internal use only.
	 *
	 * @throws UpgradeException
	 * @throws ApiRequestException
	 * @throws ApiResponseException
	 * @throws ReleaseProviderStorageException
	 * @throws ReleaseMetadataMissingException
	 * @throws ConfigCorruptionException if options are invalid
	 * @internal
	 * @ignore
	 */
	public function try_upgrade() {
		$options = get_option( self::OPTIONS_KEY );

		$should_upgrade = false;

		// Upgrade from v1 schema: 4.0.0-rc13 or earlier.
		if ( isset( $options['lockedLoadSpec'] ) || isset( $options['adminClientLoadSpec'] ) ) {
			if ( isset( $options['removeUnregisteredClients'] ) && $options['removeUnregisteredClients'] ) {
				$this->old_remove_unregistered_clients = true;
			}

			$upgraded_options = $this->convert_options_from_v1( $options );

			/**
			 * If the version is still not set for some reason, set it to a
			 * default of the latest available version.
			 */
			if ( ! isset( $upgraded_options['version'] ) ) {
				$upgraded_options['version'] = fa()->latest_version_6();
			}

			$should_upgrade = true;

			$options = $upgraded_options;
		}

		if ( ! isset( $options['dataVersion'] ) || $options['dataVersion'] < 4 ) {

			if ( ! isset( $options['compat'] ) && isset( $options['v4Compat'] ) ) {
				$v4_compat         = boolval( $options['v4Compat'] );
				$options['compat'] = $v4_compat;
				unset( $options['v4Compat'] );
			} else {
				$options['compat'] = self::DEFAULT_USER_OPTIONS['compat'];
			}

			if ( isset( $options['v4Compat'] ) ) {
				unset( $options['v4Compat'] );
			}

			$options['dataVersion'] = 4;

			$should_upgrade = true;
		}

		if ( $should_upgrade ) {
			$this->validate_options( $options );

			$this->maybe_update_last_used_release_schema_for_upgrade();

			$this->maybe_move_release_metadata_for_upgrade();

			/**
			 * Delete the main option to make sure it's removed entirely, including
			 * from the autoload cache.
			 *
			 * Function delete_option() returns false when it fails, including when the
			 * option does not exist. We know the option exists, because we just
			 * queried it above. So any other failure should halt the upgrade
			 * process to avoid inconsistent states.
			 */
			if ( ! delete_option( self::OPTIONS_KEY ) ) {
				throw UpgradeException::main_option_delete();
			}

			update_option( self::OPTIONS_KEY, $options );
		}
	}

	/**
	 * Some upgrades have involved changing how we store release metadata.
	 *
	 * If the plugin's backing data was in a valid sate before upgrade, then
	 * it should always be possible to apply any fixups to how the release
	 * metadata are stored without issuing a request to the API server for
	 * fresh metadata. Since issuing such a blocking request upon upgrade
	 * is known to cause load problems and request timeouts, let's never
	 * do it on upgrade.
	 *
	 * Internal use only.
	 *
	 * @throws ReleaseMetadataMissingException
	 * @ignore
	 * @internal
	 */
	private function maybe_move_release_metadata_for_upgrade() {
		if ( boolval( get_option( FontAwesome_Release_Provider::OPTIONS_KEY ) ) ) {
			// If this option is set, then we're all caught up.
			return;
		}

		// It used to be stored in one of these.
		$release_metadata = get_site_transient( 'font-awesome-releases' );
		if ( ! $release_metadata ) {
			$release_metadata = get_transient( 'font-awesome-releases' );
		}

		// Move it into where it belongs now.
		if ( boolval( $release_metadata ) ) {
			update_option( FontAwesome_Release_Provider::OPTIONS_KEY, $release_metadata, false );
		}

		/**
		 * Delete the old release metadata transient, if it exists.
		 * It's no longer stored as a transient.
		 */
		delete_transient( 'font-awesome-releases' );

		/**
		 * Delete the old font-awesome-last-used-release site transient, if it exists.
		 * It's no longer stored as a site (network-wide) transient.
		 */
		delete_site_transient( 'font-awesome-last-used-release' );

		/**
		 * Now we'll reset the release provider.
		 *
		 * If we've fallen through to this point, and we haven't found the release
		 * metadata stored in one of the previous locations, then this will throw an
		 * exception.
		 */
		FontAwesome_Release_Provider::reset();
	}

	/**
	 * With 4.1.0, the name of one of the keys in the LAST_USED_RELEASE_TRANSIENT changed.
	 * We can fix it up.
	 *
	 * Internal use only.
	 *
	 * @throws ReleaseMetadataMissingException
	 * @ignore
	 * @internal
	 */
	private function maybe_update_last_used_release_schema_for_upgrade() {
		$last_used_transient = get_site_transient( FontAwesome_Release_Provider::LAST_USED_RELEASE_TRANSIENT );
		if ( ! $last_used_transient ) {
			$last_used_transient = get_transient( FontAwesome_Release_Provider::LAST_USED_RELEASE_TRANSIENT );
		}

		if ( $last_used_transient && isset( $last_used_transient['use_shim'] ) ) {
			$compat = $last_used_transient['use_shim'];
			unset( $last_used_transient['use_shim'] );
			$last_used_transient['use_compatibility'] = $compat;
			set_site_transient( FontAwesome_Release_Provider::LAST_USED_RELEASE_TRANSIENT, $last_used_transient, FontAwesome_Release_Provider::LAST_USED_RELEASE_TRANSIENT_EXPIRY );
		}
	}

	/**
	 * Returns boolean indicating whether the plugin is currently configured
	 * to run the client-side conflict detection scanner.
	 *
	 * @since 4.0.0
	 * @return bool
	 */
	public function detecting_conflicts() {
		$conflict_detection = get_option( self::CONFLICT_DETECTION_OPTIONS_KEY );

		if ( isset( $conflict_detection['detectConflictsUntil'] ) && is_integer( $conflict_detection['detectConflictsUntil'] ) ) {
			return time() < $conflict_detection['detectConflictsUntil'];
		} else {
			return false;
		}
	}

	/**
	 * Returns boolean indicating whether a kit is configured.
	 *
	 * It normally shouldn't make a difference to other theme's or plugins
	 * as to whether Font Awesome is configured to use the standard CDN or a kit.
	 * Yet this is a valid way to determine that.
	 *
	 * @since 4.0.0
	 * @throws ConfigCorruptionException
	 * @return bool
	 */
	public function using_kit() {
		$options = $this->options();
		$this->validate_options( $options );
		return $this->using_kit_given_options( $options );
	}

	/**
	 * Internal use only.
	 *
	 * @internal
	 * @ignore
	 * @return bool
	 */
	private function using_kit_given_options( $options ) {
		return isset( $options['kitToken'] )
			&& isset( $options['apiToken'] )
			&& $options['apiToken']
			&& is_string( $options['kitToken'] );
	}

	/**
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	protected function stringify_constraints( $constraints ) {
		$flipped_concat_each = array_map(
			function ( $constraint ) {
				return "$constraint[1] $constraint[0]";
			},
			$constraints
		);
		return implode( ' and ', $flipped_concat_each );
	}

	/**
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	private function initialize_rest_api() {
		add_action(
			'rest_api_init',
			array(
				new FontAwesome_API_Controller( self::PLUGIN_NAME, self::REST_API_NAMESPACE ),
				'register_routes',
			)
		);
		add_action(
			'rest_api_init',
			array(
				new FontAwesome_Config_Controller( self::PLUGIN_NAME, self::REST_API_NAMESPACE ),
				'register_routes',
			)
		);
		add_action(
			'rest_api_init',
			array(
				new FontAwesome_Preference_Check_Controller( self::PLUGIN_NAME, self::REST_API_NAMESPACE ),
				'register_routes',
			)
		);
		add_action(
			'rest_api_init',
			array(
				new FontAwesome_Conflict_Detection_Controller( self::PLUGIN_NAME, self::REST_API_NAMESPACE ),
				'register_routes',
			)
		);
		add_action(
			'rest_api_init',
			array(
				new FontAwesome_V3Deprecation_Controller( self::PLUGIN_NAME, self::REST_API_NAMESPACE ),
				'register_routes',
			)
		);
	}

	/**
	 * Returns the latest available full release version of Font Awesome 5 as a string,
	 * or null if the releases metadata has not yet been successfully retrieved from the
	 * API server.
	 *
	 * As of the release of Font Awesome 6.0.0-beta1, this API is being deprecated,
	 * because the symbolic version "latest" is being deprecated. It now just means
	 * "the latest full release of Font Awesome with major version 5." Therefore,
	 * it may not be very useful any more as Font Awesome 6 is released.
	 *
	 * The recommended way to resolve the symbolic versions 'latest',
	 * '5.x', or '6