ification.Recommended
		if ( ! isset( $_REQUEST[ self::ADMIN_TAB_QUERY_VAR ] ) || empty( $_REQUEST[ self::ADMIN_TAB_QUERY_VAR ] ) ) {
			return null;
		}

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$value = $_REQUEST[ self::ADMIN_TAB_QUERY_VAR ];

		// These values are defined in the Redux reducer module of the admin JS React app.
		switch ( $value ) {
			case 'ts':
				return 'ADMIN_TAB_TROUBLESHOOT';
			case 's':
				return 'ADMIN_TAB_SETTINGS';
			default:
				return null;
		}
	}

	/**
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	private function emit_v3_deprecation_admin_notice( $data ) {
		?>
		<div class="notice notice-warning is-dismissible">
			<p>
				<?php esc_html_e( 'Hey there, from the Font Awesome plugin!', 'font-awesome' ); ?>
			</p>
			<p>
				<?php
					printf(
						/* translators: 1: detected icon name 2: literal icon shortcode */
						esc_html__(
							'Looks like you\'re using an %2$s shortcode with an old Font Awesome 3 icon name: %1$s. We\'re phasing those out, so it will stop working on your site soon.',
							'font-awesome'
						),
						'<code>' . esc_html( $data['atts']['name'] ) . '</code>',
						'<code>[icon]</code>'
					);
				?>
			</p>
			<p>
				<?php
					printf(
						/* translators: 1: opening anchor tag with url 2: closing anchor tag */
						esc_html__(
							'Head over to the %1$sFont Awesome Settings%2$s page to see how you can fix it up, or snooze this warning for a while.',
							'font-awesome'
						),
						'<a href="' . esc_html( $this->settings_page_url() ) . '&tab=ts">',
						'</a>'
					);
				?>
			</p>
		</div>
		<?php
	}

	/**
	 * Initalizes everything about the admin environment except the React app
	 * bundle, which is handled in maybe_enqueue_admin_assets().
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 */
	public function initialize_admin() {
		$v3deprecation_warning_data = $this->get_v3deprecation_warning_data();

		if ( $v3deprecation_warning_data && ! ( isset( $v3deprecation_warning_data['snooze'] ) && $v3deprecation_warning_data['snooze'] ) ) {

			$v3_deprecation_command = new FontAwesome_Command(
				function() use ( $v3deprecation_warning_data ) {
					$current_screen = get_current_screen();
					if ( $current_screen && fa()->screen_id !== $current_screen->id ) {
						fa()->emit_v3_deprecation_admin_notice( $v3deprecation_warning_data );
					}
				}
			);

			add_action(
				'admin_notices',
				array( $v3_deprecation_command, 'run' )
			);
		}

		$admin_menu_command = new FontAwesome_Command(
			function() {
				fa()->screen_id = add_options_page(
					/* translators: add_options_page page_title */
					esc_html__( 'Font Awesome Settings', 'font-awesome' ),
					/* translators: add_options_page menu_title */
					esc_html__( 'Font Awesome', 'font-awesome' ),
					'manage_options',
					self::OPTIONS_PAGE,
					array( fa(), 'create_admin_page' )
				);
			}
		);

		add_action(
			'admin_menu',
			array( $admin_menu_command, 'run' )
		);

		$plugin_action_links_command = new FontAwesome_Command(
			function ( $links ) {
				$mylinks = array(
					/* translators: label for link to settings page on plugin listing */
					'<a href="' . fa()->settings_page_url() . '">' . esc_html__( 'Settings', 'font-awesome' ) . '</a>',
				);
				return array_merge( $links, $mylinks );
			}
		);

		add_filter(
			'plugin_action_links_' . FONTAWESOME_PLUGIN_FILE,
			array( $plugin_action_links_command, 'run' )
		);

		$multi_version_warning_command = new FontAwesome_Command(
			function( $plugin_file, $plugin_data, $status ) {
				if ( version_compare( FontAwesome::PLUGIN_VERSION, $plugin_data['Version'], 'ne' ) ) {
					$loader_version = FontAwesome_Loader::instance()->loaded_path();
					?>
					<tr>
						<td>&nbsp;</td>
						<td colspan="2" class="notice notice-info notice-alt">
							<p>
								<b><?php esc_html_e( 'Great Scott!', 'font-awesome' ); ?></b>
								<?php
									printf(
										/* translators: 1: path to plugin or theme code file 2: current Font Awesome plugin version number */
										esc_html__(
											'The active version of the Font Awesome plugin is being loaded by this plugin or theme: %1$s since it\'s the newest (%2$s). We recommend you update the plugin above to the latest version. In the meantime, we\'ll use that newer version for editing your Font Awesome settings so you\'ll be sure to hit 88mph with those icons.',
											'font-awesome'
										),
										'<code>' . esc_html( $loader_version ) . '</code>',
										'<b>ver. ' . esc_html( FontAwesome::PLUGIN_VERSION ) . '</b>'
									);
								?>
							</p>
							<p>
								<?php
									esc_html_e( 'You\'ve got more than one version of the Font Awesome plugin installed.', 'font-awesome' );
								?>
							</p>
						</td>
					</tr>
					<?php
				}
			}
		);

		add_action(
			'after_plugin_row_' . FONTAWESOME_PLUGIN_FILE,
			array( $multi_version_warning_command, 'run' ),
			10,
			3
		);
	}

	/**
	 * Returns current options.
	 *
	 * Internal use only. Not part of this plugin's public API.
	 *
	 * @throws ConfigCorruptionException
	 * @internal
	 * @ignore
	 * @return array
	 */
	public function options() {
		$options = get_option( self::OPTIONS_KEY );

		if ( ! $options ) {
			throw new ConfigCorruptionException();
		}

		return $options;
	}

	/**
	 * Validates options.
	 *
	 * Internal use only. Not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 * @throws ConfigCorruptionException if options are invalid
	 */
	public function validate_options( $options ) {
		$using_kit = $this->using_kit_given_options( $options );
		$kit_token = isset( $options['kitToken'] ) ? $options['kitToken'] : null;
		$api_token = isset( $options['apiToken'] ) ? $options['apiToken'] : null;
		$version   = isset( $options['version'] ) ? $options['version'] : null;

		if ( ! isset( $options['usePro'] ) || ! is_bool( $options['usePro'] ) ) {
			throw new ConfigCorruptionException();
		}

		if ( $using_kit ) {
			if ( ! boolval( $api_token ) ) {
				throw new ConfigCorruptionException();
			}

			if ( ! is_string( $kit_token ) ) {
				throw new ConfigCorruptionException();
			}

			if ( ! is_string( $version ) ) {
				throw new ConfigCorruptionException();
			}
		} else {
			/**
			 * If we're not using a kit, then the version cannot be "latest",
			 * "5.x", or "6.x" at this point. It must have already been resolved
			 * into a concrete version.
			 */
			$version_is_concrete = is_string( $version )
				&& 1 === preg_match( '/^[0-9]+\.[0-9]+\.[0-9]+/', $version );

			if ( ! $version_is_concrete ) {
				throw new ConfigCorruptionException();
			}

			$version_is_v6 = is_string( $version )
				&& 1 === preg_match( '/^6\./', $version );

			$is_pro = boolval( $options['usePro'] );

			/**
			 * Pro Version 6 CDN is not supported.
			 */
			if ( $version_is_v6 && $is_pro ) {
				throw new ConfigCorruptionException();
			}
		}

		if ( ! isset( $options['compat'] ) || ! is_bool( $options['compat'] ) ) {
			throw new ConfigCorruptionException();
		}

		if ( ! isset( $options['pseudoElements'] ) || ! is_bool( $options['pseudoElements'] ) ) {
			throw new ConfigCorruptionException();
		}

		if (
			! isset( $options['technology'] ) ||
			! is_string( $options['technology'] ) ||
			false === array_search( $options['technology'], array( 'svg', 'webfont' ), true )
		) {
			throw new ConfigCorruptionException();
		}
	}

	/**
	 * An array of md5 hashes that identify detected conflicting versions of
	 * Font Awesome that the site owner has chosen to block from being enqueued.
	 *
	 * It is managed through the plugin's settings page.
	 *
	 * @since 4.0.0
	 * @return array
	 */
	public function blocklist() {
		$conflict_detection = get_option( self::CONFLICT_DETECTION_OPTIONS_KEY );