abase.
	 *
	 * Returns 0 if that value is unset in the db.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 * @return integer
	 */
	protected function detect_conflicts_until() {
		$conflict_detection = get_option(
			self::CONFLICT_DETECTION_OPTIONS_KEY,
			self::DEFAULT_CONFLICT_DETECTION_OPTIONS
		);

		return isset( $conflict_detection['detectConflictsUntil'] )
			? $conflict_detection['detectConflictsUntil'] : 0;
	}

	/**
	 * Converts a given options array with a v1 schema to one with a v2 schema.
	 * There are significant changes from the schema used by 4.0.0-rc9 and before.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 * @param $options
	 * @return array
	 */
	public function convert_options_from_v1( $options ) {
		$converted_options = self::DEFAULT_USER_OPTIONS;

		if ( isset( $options['usePro'] ) ) {
			$converted_options['usePro'] = $options['usePro'];
		} else {
			$converted_options['usePro'] = self::DEFAULT_USER_OPTIONS['usePro'];
		}

		if ( isset( $options['version'] ) ) {
			$converted_options['version'] = $options['version'];
		}

		if ( isset( $options['lockedLoadSpec'] ) ) {
			$converted_options['technology'] = isset( $options['lockedLoadSpec']['method'] )
				? $options['lockedLoadSpec']['method']
				: 'webfont';

			/**
			 * If technology is webfont, always coerce pseudo-elements to true.
			 * Otherwise, carry over whatever value it had before.
			 */
			$converted_options['pseudoElements'] =
				'webfont' === $converted_options['technology']
					? true
					: (
						isset( $options['lockedLoadSpec']['pseudoElements'] )
							? $options['lockedLoadSpec']['pseudoElements']
							: false
					);

			$converted_options['compat'] = $options['lockedLoadSpec']['v4shim'];
		} elseif ( isset( $options['adminClientLoadSpec'] ) ) {
			$converted_options['technology'] = $options['adminClientLoadSpec']['method'];

			$converted_options['pseudoElements'] = 'svg' === $options['adminClientLoadSpec']['method']
				&& $options['adminClientLoadSpec']['pseudoElements'];

			$converted_options['compat'] = $options['adminClientLoadSpec']['v4shim'];
		}

		return $converted_options;
	}

	/**
	 * Callback function for creating the plugin's admin page.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 */
	public function create_admin_page() {
		include_once FONTAWESOME_DIR_PATH . 'admin/views/main.php';
	}

	/**
	 * Resets the singleton instance referenced by this class.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @ignore
	 * @internal
	 * @return FontAwesome
	 */
	public static function reset() {
		self::$instance = null;
		return fa();
	}

	/**
	 * Triggers the font_awesome_preferences action to gather preferences from clients.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 * @throws PreferenceRegistrationException
	 */
	public function gather_preferences() {
		/**
		 * Fired when the plugin is ready for clients to register their preferences.
		 *
		 * @since 4.0.0
		 */
		try {
			do_action( 'font_awesome_preferences' );
		} catch ( Exception $e ) {
			throw PreferenceRegistrationException::with_thrown( $e );
		} catch ( Error $e ) {
			throw PreferenceRegistrationException::with_thrown( $e );
		}
	}

	/**
	 * Returns current preferences conflicts, keyed by option name.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * Should normally only be called after the `font_awesome_enqueued` action has triggered, indicating that all
	 * client preferences have been registered and processed.
	 *
	 * The returned array includes all conflicts between the options configured for this plugin by the site owner
	 * and any preferences registered by themes or plugins.
	 *
	 * The presence of conflicts will not stop this plugin from loading Font Awesome according to its
	 * configured options, but they will be presented to the site owner in the plugin's admin settings page to
	 * aid in troubleshooting.
	 *
	 * @ignore
	 * @internal
	 * @param $options options to use for comparison. Uses $this->options() by default.
	 * @see FontAwesome::register() register() documents all client preference keys
	 * @return array
	 */
	public function conflicts_by_option( $options = null ) {
		$conflicts = array();

		$options_for_comparison = is_null( $options ) ? $this->options() : $options;

		foreach ( $this->conflicts_by_client( $options_for_comparison ) as $client_