

/**
 * Enrich image information with extra data we can retrieve from the media library.
 * Add missing `width`, `height`, `srcset`, `sizes`, `title`, `alt` and `caption` properties to the image.
 *
 * @param array $media_file  - An array containing the media attributes for a specific image.
 *
 * @returns array $media_file_enriched
 */
function enrich_image_meta( $media_file ) {
	$attachment_id = isset( $media_file['id'] ) ? $media_file['id'] : null;
	$image         = wp_get_attachment_image_src( $attachment_id, 'full', false );
	if ( ! $image ) {
		return $media_file;
	}
	list( $src, $width, $height ) = $image;
	// Bail if url stored in block attributes is different than the media library one for that id.
	if ( isset( $media_file['url'] ) && ! is_same_resource( $media_file['url'], $src ) ) {
		return $media_file;
	}
	$image_meta = wp_get_attachment_metadata( $attachment_id );
	if ( ! is_array( $image_meta ) ) {
		return $media_file;
	}
	$size_array = array( absint( $width ), absint( $height ) );
	return array_merg