lled;
			}
			$plugin_id = self::get_plugin_id_by_slug( $slug );
		} elseif ( self::is_plugin_active( $plugin_id ) ) {
			return true; // Already installed and active.
		}

		if ( ! current_user_can( 'activate_plugins' ) ) {
			return new WP_Error( 'not_allowed', __( 'You are not allowed to activate plugins on this site.', 'jetpack-plugins-installer' ) );
		}
		$activated = activate_plugin( $plugin_id );
		if ( is_wp_error( $activated ) ) {
			return $activated;
		}

		return true;
	}

	/**
	 * Install a plugin.
	 *
	 * @since-jetpack 5.8.0
	 *
	 * @param string $slug Plugin slug.
	 *
	 * @return bool|WP_Error True if installation succeeded, error object otherwise.
	 */
	public static function install_plugin( $slug ) {
		if ( is_multisite() && ! current_user_can( 'manage_network' ) ) {
			return new WP_Error( 'not_allowed', __( 'You are not allowed to install plugins on this site.', 'jetpack-plugins-installer' ) );
		}

		$skin     = new Automatic_Install_Skin();
		$upgrader = new Plugin_Upgrader( $skin );
		$zip_url  = self::generate_wordpress_org_plugin_download_link( $slug );
		$mc_stats = new A8c_Mc_Stats();

		$result = $upgrader->install( $zip_url );

		if ( is_wp_error( $result ) ) {
			$mc_stats->add( 'install-plugin', "fail-$slug" );
			return $result;
		}

		$plugin     = self::get_plugin_id_by_slug( $slug );
		$error_code = 'install_error';
		if ( ! $plugin ) {
			$error = __( 'There was an error installing your plugin', 'jetpack-plugins-installer' );
		}

		if ( ! $result ) {
			$error_code = $upgrader->skin->get_main_error_code();
			$message    = $upgrader->skin->get_main_error_message();
			$error      = $message ? $message : __( 'An unknown error occurred during installation', 'jetpack-plugins-installer' );
		}

		if ( ! empty( $error ) ) {
			if ( 'download_failed' === $error_code ) {
				// For backwards compatibility: versions prior to 3.9 would return no_package instead of download_failed.
				$error_code = 'no_package';
			}

			$mc_stats->add( 'install-plugin', "fail-$slug" );
			return new WP_Error( $error_code, $error, 400 );
		}

		$mc_stats->add( 'install-plugin', "success-$slug" );
		return (array) $upgrader->skin->get_upgrade_messages();
	}

	/**
	 * Get WordPress.org zip download link from a plugin slug
	 *
	 * @param string $plugin_slug Plugin slug.
	 */
	protected static function generate_wordpress_org_plugin_download_link( $plugin_slug ) {
		return "https://downloads.wordpress.org/plugin/$plugin_slug.latest-stable.zip";
	}

	/**
