			if ( $done ) {
					$enabled = true;
				}

				$connection_list[] = array(
					'unique_id'       => $unique_id,
					'service_name'    => $service_name,
					'service_label'   => $this->get_service_label( $service_name ),
					'display_name'    => $this->get_display_name( $service_name, $connection ),
					'profile_picture' => $this->get_profile_picture( $connection ),

					'enabled'         => $enabled,
					'done'            => $done,
					'toggleable'      => $toggleable,
					'global'          => 0 == $connection_data['user_id'], // phpcs:ignore Universal.Operators.StrictComparisons.LooseEqual,WordPress.PHP.StrictComparisons.LooseComparison -- Other types can be used at times.
				);
			}
		}

		return $connection_list;
	}

	/**
	 * Checks if post has already been shared by Publicize in the past.
	 *
	 * @since 0.1.0
	 * @since-jetpack 6.7.0
	 *
	 * @param integer $post_id Optional. Post ID to query connection status for: will use current post if missing.
	 *
	 * @return bool True if post has already been shared by Publicize, false otherwise.
	 */
	abstract public function post_is_done_sharing( $post_id = null );

	/**
	 * Retrieves full list of available Publicize connection services.
	 *
	 * Retrieves current available publicize service connections
	 * with associated labels and URLs.
	 *
	 * @since 0.1.0
	 * @since-jetpack 6.7.0
	 *
	 * @return array {
	 *     Array of UI service connection data for all services
	 *
	 *     @type string 'name'  Name of service.
	 *     @type string 'label' Display label for service.
	 *     @type string 'url'   URL for adding connection to service.
	 * }
	 */
	public function get_available_service_data() {
		$available_services     = $this->get_services( 'all' );
		$available_service_data = array();

		foreach ( $available_services as $service_name => $service ) {
			$available_service_data[] = array(
				'name'  => $service_name,
				'label' => $this->get_service_label( $service_name ),
				'url'   => $this->connect_url( $service_name ),
			);
		}

		return $available_service_data;
	}

	/**
	 * Site Data
	 */

	/**
	 * Get user ID.
	 *
	 * @return int The current user's ID, or 0 if no user is logged in.
	 */
	public function user_id() {
		return get_current_user_id();
	}

	/**
	 * Get site ID.
	 *
	 * @return int Site ID.
	 */
	public function blog_id() {
		return get_current_blog_id();
	}

	/**
	 * Posts
	 */

	/**
	 * Checks old and new status to see if the post should be flagged as
	 * ready to Publicize.
	 *
	 * Attached to the `transition_post_status` filter.
	 *
	 * @param string  $new_status New status.
	 * @param string  $old_status Old status.
	 * @param WP_Post $post Post object.
	 * @return void
	 */
	abstract public function flag_post_for_publicize( $new_status, $old_status, $post );

	/**
	 * Ensures the Post internal post-type supports `publicize`
	 *
	 * This feature support flag is used by the REST API.
	 */
	public function add_post_type_support() {
		add_post_type_support( 'post', 'publicize' );
	}

	/**
	 * Can the current user access Publicize Data.
	 *
	 * @param int $post_id 0 for general access. Post_ID for specific access.
	 * @return bool
	 */
	public function current_user_can_access_publicize_data( $post_id = 0 ) {
		/**
		 * Filter what user capability is required to use the publicize form on the edit post page. Useful if publish post capability has been removed from role.
		 *
		 * @since 0.1.0
		 * @since-jetpack 4.1.0
		 *
		 * @param string $capability User capability needed to use publicize
		 */
		$capability = apply_filters( 'jetpack_publicize_capability', 'publish_posts' );

		if ( 'publish_posts' === $capability && $post_id ) {
			return current_user_can( 'publish_post', $post_id );
		}

		return current_user_can( $capability );
	}

	/**
	 * Auth callback for the protected ->POST_MESS post_meta
	 *
	 * @param int $object_id Post ID.
	 * @return bool
	 */
	public function message_meta_auth_callback( $object_id ) {
		return $this->current_user_can_access_publicize_data( $object_id );
	}

	/**
	 * Registers the post_meta for use in the REST API.
	 *
	 * Registers for each post type that with `publicize` feature support.
	 */
	public function register_post_meta() {
		$message_args = array(
			'type'          => 'string',
			'description'   => __( 'The message to use instead of the title when sharing to Publicize Services', 'jetpack-publicize-pkg' ),
			'single'        => true,
			'default'       => '',
			'show_in_rest'  => array(
				'name' => 'jetpack_publicize_message',
			),
			'auth_callback' => array( $this, 'message_meta_auth_callback' ),
		);

		$tweetstorm_args = array(
			'type'          => 'boolean',
			'description'   => __( 'Whether or not the post should be treated as a Twitter thread.', 'jetpack-publicize-pkg' ),
			'single'        => true,
			'default'       => false,
			'show_in_rest'  => array(
				'name' => 'jetpack_is_tweetstorm',
			),
			'auth_callback' => array( $this, 'message_meta_auth_callback' ),
		);

		$publicize_feature_enable_args = array(
			'type'          => 'boolean',
			'description'   => __( 'Whether or not the Share Post feature is enabled.', 'jetpack-publicize-pkg' ),
			'single'        => true,
			'default'       => true,
			'show_in_rest'  => array(
				'name' => 'jetpack_publicize_feature_enabled',
			),
			'auth_callback' => array( $this, 'message_meta_auth_callback' ),
		);

		foreach ( get_post_types() as $post_type ) {
			if ( ! $this->post_type_is_publicizeable( $post_type ) ) {
				continue;
			}

			$message_args['object_subtype']                  = $post_type;
			$tweetstorm_args['object_subtype']               = $post_type;
			$publicize_feature_enable_args['object_subtype'] = $post_type;

			register_meta( 'post', $this->POST_MESS, $message_args );
			register_meta( 'post', $this->POST_TWEETSTORM, $tweetstorm_args );
			register_meta( 'post', self::POST_PUBLICIZE_FEATURE_ENABLED, $publicize_feature_enable_args );
		}
	}

	/**
	 * Helper function to allow us to not publicize posts in certain contexts.
	 *
	 * @param WP_Post $post Post object.
	 */
	public function should_submit_post_pre_checks( $post ) {
		$submit_post = true;

		if ( defined( 'WP_IMPORTING' ) && WP_IMPORTING ) {
			$submit_post = false;
		}

		if (
			defined( 'DOING_AUTOSAVE' )
		&&
			DOING_AUTOSAVE
		) {
			$submit_post = false;
		}

		// To stop quick edits from getting publicized.
		if ( did_action( 'wp_ajax_inline-save' ) ) {
			$submit_post = false;
		}

		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		if ( ! empty( $_GET['bulk_edit'] ) ) {
			$submit_post = false;
		}
		// phpcs:enable WordPress.Security.NonceVerification.Recommended

		// - API/XML-RPC Test Posts
		if (
			(
				defined( 'XMLRPC_REQUEST' )
			&&
				XMLRPC_REQUEST
			||
				defined( 'APP_REQUEST' )
			&&
				APP_REQUEST
			)
		&&
			0 === strpos( $post->post_title, 'Temporary Post Used For Theme Detection' )
		) {
			$submit_post = false;
		}

		// Only work with certain statuses (avoids inherits, auto drafts etc).
		if ( ! in_array( $post->post_status, array( 'publish', 'draft', 'future' ), true ) ) {
			$submit_post = false;
		}

		// Don't publish password protected posts.
		if ( '' !== $post->post_password ) {
			$submit_post = false;
		}

		return $submit_post;
	}

	/**
	 * Fires when a post is saved, checks conditions and saves state in postmeta so that it
	 * can be picked up later by @see ::publicize_post() on WordPress.com codebase.
	 *
	 * Attached t