censing/user/activation-notice-dismiss',
			array(
				'methods'             => \WP_REST_Server::EDITABLE,
				'callback'            => __CLASS__ . '::update_licensing_activation_notice_dismiss',
				'permission_callback' => __CLASS__ . '::user_licensing_permission_check',
				'args'                => array(
					'last_detached_count' => array(
						'required'          => true,
						'type'              => 'integer',
						'validate_callback' => __CLASS__ . '::validate_non_neg_int',
					),
				),
			)
		);

		/**
		 * Attach licenses to user account
		 */
		register_rest_route(
			'jetpack/v4',
			'/licensing/attach-licenses',
			array(
				'methods'             => \WP_REST_Server::EDITABLE,
				'callback'            => __CLASS__ . '::attach_jetpack_licenses',
				'permission_callback' => __CLASS__ . '::user_licensing_permission_check',
				'args'                => array(
					'licenses' => array(
						'required' => true,
						'type'     => 'array',
						'items'    => array(
							'type' => 'string',
						),
					),
				),
			)
		);
	}

	/**
	 * Verify that the user can set a Jetpack license key
	 *
	 * @since 1.7.0
	 *
	 * @since-jetpack 9.5.0
	 *
	 * @return bool|WP_Error True if user is able to set a Jetpack license key
	 */
	public static function set_jetpack_license_key_permission_check() {
		if ( Licensing::instance()->is_licensing_input_enabled() ) {
			return true;
		}

		return new WP_Error( 'invalid_user_permission_set_jetpack_license_key', self::$user_permissions_error_msg, array( 'status' => rest_authorization_required_code() ) );

	}

	/**
	 * Verify that user can view and update user-licensing data.
	 *
	 * @since 1.7.0
	 *
	 * @return bool Whether the user is currently connected and they are the connection owner.
	 */
	public static function user_licensing_permission_check() {
		$connection_manager = new Connection_Manager( 'jetpack' );

		if ( $connection_manager->is_user_connected() && $connection_manager->is_connection_owner() ) {
			return true;
		}

		return new WP_Error( 'invalid_permission_manage_user_licenses', self::$user_permissions_error_msg, array( 'status' => rest_authorization_required_code() ) );
	}

	/**
	 * Verify that user can manage_options
	 *
	 * @since 1.7.0
	 *
	 * @return bool Whether user has the capability 'manage_options'.
	 */
	public static function can_manage_options_check() {
		if ( current_user_can( 'manage_options' ) ) {
			return true;
		}

		return new WP_Error( 'invalid_user_permission_view_admin', self::$user_permissions_error_msg, array( 'status' => rest_authorization_required_code() ) );
	}

	/**
	 * Validates that the parameter is a string.
	 *
	 * @since 1.7.0
	 *
	 * @since-jetpack 4.3.0
	 *
	 * @param string          $value Value to check.
	 * @param WP_REST_Request $request The request sent to the WP REST API.
	 * @param string          $param Name of the parameter passed to endpoint holding $value.
	 *
	 * @return bool|WP_Error
	 */
	public static function validate_string( $value, $request, $param ) {
		if ( ! is_string( $value ) ) {
			/* translators: %s: The literal parameter name. Should not be translated. */
			return new WP_Error( 'invalid_param', sprintf( esc_html__( '%s must be a string.', 'jetpack-licensing' ), $param ) );
		}
		return true;
	}

	/**
	 * Validates that the parameter is a non-negative integer (includes 0).
	 *
	 * @since 1.7.0
	 *
	 * @since-jetpack 10.4.0
	 *
	 * @param int             $value Value to check.
	 * @param WP_REST_Request $request The request sent to the WP REST API.
	 * @param string          $param Name of the parameter passed to endpoint holding $value.
	 *
	 * @return bool|WP_Error
	 */
	public static function validate_non_neg_int( $value, $request, $param ) {
		if ( ! is_numeric( $value ) || $value < 0 ) {
			return new WP_Error(
				'invalid_param',
				/* translators: %s: The literal parameter name. Should not be translated. */
				sprintf( esc_html__( '%s must