age_other( 'linkedin' );
	}

	/**
	 * Save the options page for Twitter.
	 */
	public function options_save_twitter() {
		$this->options_save_other( 'twitter' );
	}

	/**
	 * Save the options page for LinkedIn.
	 */
	public function options_save_linkedin() {
		$this->options_save_other( 'linkedin' );
	}

	/**
	 * Save the options page for a service.
	 *
	 * @param string $service_name Name of the service to save options for.
	 */
	public function options_save_other( $service_name ) {
		$connection_name = isset( $_REQUEST['connection'] ) ? filter_var( wp_unslash( $_REQUEST['connection'] ) ) : '';

		// Nonce check.
		check_admin_referer( 'save_' . $service_name . '_token_' . $connection_name );

		$this->globalization( $connection_name );
	}

	/**
	 * If there's only one shared connection to Twitter set it as twitter:site tag.
	 *
	 * @param string $tag Tag.
	 */
	public function enhaced_twitter_cards_site_tag( $tag ) {
		$custom_site_tag = get_option( 'jetpack-twitter-cards-site-tag' );
		if ( ! empty( $custom_site_tag ) ) {
			return $tag;
		}
		if ( ! $this->is_enabled( 'twitter' ) ) {
			return $tag;
		}
		$connections = $this->get_connections( 'twitter' );
		foreach ( $connections as $connection ) {
			$connection_meta = $this->get_connection_meta( $connection );
			if ( $this->is_global_connection( $connection_meta ) ) {
				// If the connection is shared.
				return $this->get_display_name( 'twitter', $connection );
			}
		}

		return $tag;
	}

	/**
	 * Save the Publicized Twitter account when publishing a post.
	 *
	 * @param bool   $submit_post Should the post be publicized.
	 * @param int    $post_id Post ID.
	 * @param string $service_name Service name.
	 * @param array  $connection Array of connection details.
	 */
	public function save_publicized_twitter_account( $submit_post, $post_id, $service_name, $connection ) {
		if ( 'twitter' === $service_name && $submit_post ) {
			$connection_meta        = $this->get_connection_meta( $connection );
			$publicize_twitter_user = get_post_meta( $post_id, '_publicize_twitter_user' );
			if ( empty( $publicize_twitter_user ) || ! $this->is_global_connection( $connection_meta ) ) {
				update_post_meta( $post_id, '_publicize_twitter_user', $this->get_display_name( 'twitter', $connection ) );
			}
		}
	}

	/**
	 * Get the Twitter username.
	 *
	 * @param string $account Twitter username.
	 * @param int    $post_id ID of the post.
	 * @return string
	 */
	public function get_publicized_twitter_account( $account, $post_id ) {
		if ( ! empty( $account ) ) {
			return $account;
		}
		$account = get_post_meta( $post_id, '_publicize_twitter_user', true );
		if ( ! empty( $account ) ) {
			return $account;
		}

		return '';
	}

	/**
	 * Save the Publicized Facebook account when publishing a post
	 * Use only Personal accounts, not Facebook Pages
	 *
	 * @param bool   $submit_post Should the post be publicized.
	 * @param int    $post_id Post ID.
	 * @param string $service_name Service name.
	 * @param array  $connection Array of connection details.
	 */
	public function save_publicized_facebook_account( $submit_post, $post_id, $service_name, $connection ) {
		$connection_meta = $this->get_connection_meta( $connection );
		if ( 'facebook' === $service_name && isset( $connection_meta['connection_data']['meta']['facebook_profile'] ) && $submit_post ) {
			$publicize_facebook_user = get_post_meta( $post_id, '_publicize_