
			$this->set_next_sync_time( time() + $this->get_sync_wait_time(), $queue->id );
		}

		return $sync_result;
	}

	/**
	 * Retrieve the next sync items to send.
	 *
	 * @access public
	 *
	 * @param (array|Automattic\Jetpack\Sync\Queue_Buffer) $buffer_or_items Queue buffer or array of objects.
	 * @param boolean                                      $encode Whether to encode the items.
	 * @return array Sync items to send.
	 */
	public function get_items_to_send( $buffer_or_items, $encode = true ) {
		// Track how long we've been processing so we can avoid request timeouts.
		$start_time    = microtime( true );
		$upload_size   = 0;
		$items_to_send = array();
		$items         = is_array( $buffer_or_items ) ? $buffer_or_items : $buffer_or_items->get_items();
		if ( ! is_array( $items ) ) {
			$items = array();
		}

		// Set up current screen to avoid errors rendering content.
		require_once ABSPATH . 'wp-admin/includes/class-wp-screen.php';
		require_once ABSPATH . 'wp-admin/includes/screen.php';
		set_current_screen( 'sync' );
		$skipped_items_ids = array();
		/**
		 * We estimate the total encoded size as we go by encoding each item individually.
		 * This is expensive, but the only way to really know :/
		 */
		foreach ( $items as $key => $item ) {
			// Suspending cache addition help prevent overloading in memory cache of large sites.
			wp_suspend_cache_addition( true );
			/**
			 * Modify the data within an action before it is serialized and sent to the server
			 * For example, during full sync this expands Post ID's into full Post objects,
			 * so that we don't have to serialize the whole object into the queue.
			 *
			 * @since 1.6.3
			 * @since-jetpack 4.2.0
			 *
			 * @param array The action parameters
			 * @param int The ID of the user who triggered the action
			 */
			$item[1] = apply_filters( 'jetpack_sync_before_send_' . $item[0], $item[1], $item[2] );
			wp_suspend_cache_addition( false );
			// Serialization usage can lead to empty, null or false action_name. Lets skip as there is no information to send.
			if ( empty( $item[0] ) || false === $item[1] ) {
				$skipped_items_ids[] = $key;
				continue;
			}
			$encoded_item = $this->codec->encode( $item );
			$upload_size += strlen( $encoded_item );
			if ( $upload_size > $this->upload_max_bytes && count( $items_to_send ) > 0 ) {
				break;
			}
			$items_to_send[ $key ] = $encode ? $encoded_item : $item;
			if ( microtime( true ) - $start_time > $this->max_dequeue_time ) {
				break;
			}
		}

		return array( $items_to_send, $skipped_items_ids, $items, microtime( true ) - $start_time );
	}

	/**
	 * If supported, flush all response data to the client and finish the request.
	 * This allows for time consuming tasks to be performed without leaving the connection open.
	 *
	 * @access p