-response-status',
		'target-type',
		'timeline-pillar',
		'timeline-type',

	); // end taxonomies.

	/**
	 * List of options to sync
	 *
	 * @access private
	 * @static
	 *
	 * @var array
	 */
	private static $options_to_sync = array(
		'jetpack_search_color_theme',
		'jetpack_search_result_format',
		'jetpack_search_default_sort',
		'jetpack_search_overlay_trigger',
		'jetpack_search_excluded_post_types',
		'jetpack_search_highlight_color',
		'jetpack_search_enable_sort',
		'jetpack_search_inf_scroll',
		'jetpack_search_show_powered_by',
		'instant_search_enabled',
	); // end options.

	/*
	 * Taxonomies we know don't sync.
	 * See also sync/src/class-defaults.php
	 *
	 * 'network'
	 * 'post_status'
	 * 'product_cat'
	 * 'tags'
	 *
	 */

	//
	// Hooks into sync.

	/**
	 * Add Search post meta to the post meta whitelist.
	 *
	 * @param array $list Existing post meta whitelist.
	 * @return array Updated post meta whitelist.
	 */
	public function add_search_post_meta_whitelist( $list ) {
		return array_merge( $list, $this->get_all_postmeta_keys() );
	}

	/**
	 * Add Search options to the options whitelist.
	 *
	 * @param array $list Existing options whitelist.
	 * @return array Updated options whitelist.
	 */
	public function add_search_options_whitelist( $list ) {
		return array_merge( $list, $this->get_all_option_keys() );
	}

	//
	// Indexing functions for wp.com.

	/**
	 *
	 * Check whether a postmeta or taxonomy 'key' is in the indexable
	 * list. This is called by the indexing code on wp.com to decide
	 * whether to include something in the index.
	 *
	 * @static
	 * @access public
	 *
	 * @param string $type Either 'postmeta' or 'taxonomy'.
	 * @param string $key The postmeta key or taxonomy name.
	 * @return boolean
	 */
	public static function is_indexable( $type, $key ) {
		switch ( $type ) {
			case 'postmeta':
				return isset( self::$postmeta_to_sync[ $key ] );
			case 'taxonomy':
				return in_array( $key, self::$taxonomies_to_sync, true );
		}
		return false;
	}

	/**
	 *
	 * Get the indexing spec for a postmeta key.
	 *
	 * @static
	 * @access public
	 *
	 * @param string $key The postmeta key.
	 * @return array The spec.
	 */
	public static function get_postmeta_spec( $key ) {
		return self::$postmeta_to_sync[ $key ];
	}

	/**
	 * Get all post meta keys that get synced.
	 *
	 * @access public
	 *
	 * @return array List of post meta keys that get synced.
	 */
	public static function get_all_postmeta_keys() {
		return array_keys( self::$postmeta_to_sync );
	}

	/**
	 * Get all option keys that get synced.
	 *
	 * @access public
	 *
	 * @return array List of option keys that get synced.
	 */
	public static function get_all_option_keys() {
		return self::$options_to_sync;
	}

	/**
	 * Get all unindexed postmeta.
	 *  This is mostly for testing.
	 *
	 * @access public
	 *
	 * @return array List of postmeta that are not synced.
	 */
	public static function get_all_unindexed_postmeta_keys() {
		return array_keys( self::$unindexed_postmeta );
	}

	/*