f ( function_exists( 'ignore_user_abort' ) ) {
			ignore_user_abort( true );
		}

		/* Don't make the request block till we finish, if possible. */
		if ( Constants::is_true( 'REST_REQUEST' ) || Constants::is_true( 'XMLRPC_REQUEST' ) ) {
			$this->fastcgi_finish_request();
		}

		$checkout_start_time = microtime( true );

		$buffer = $queue->checkout_with_memory_limit( $this->dequeue_max_bytes, $this->upload_max_rows );

		if ( ! $buffer ) {
			// Buffer has no items.
			return new WP_Error( 'empty_buffer' );
		}

		if ( is_wp_error( $buffer ) ) {
			return $buffer;
		}

		$checkout_duration = microtime( true ) - $checkout_start_time;

		list( $items_to_send, $skipped_items_ids, $items, $preprocess_duration ) = $this->get_items_to_send( $buffer, true );
		if ( ! empty( $items_to_send ) ) {
			/**
			 * Fires when data is ready to send to the server.
			 * Return false or WP_Error to abort the sync (e.g. if there's an error)
			 * The items will be automatically re-sent later
			 *
			 * @since 1.6.3
			 * @since-jetpack 4.2.0
			 *
			 * @param array  $data The action buffer
			 * @param string $codec The codec name used to encode the data
			 * @param double $time The current time
			 * @param string $queue The queue used to send ('sync' or 'full_sync')
			 * @param float  $checkout_duration The duration of the checkout operation.
			 * @param float  $preprocess_duration The duration of the pre-process operation.
			 * @param int    $queue_size The size of the sync queue at the time of processing.
			 */
			Settings::set_is_sending( true );
			$processed_item_ids = apply_filters( 'jetpack_sync_send_data', $items_to_send, $this->codec->name(), microtime( true ), $queue->id, $checkout_duration, $preprocess_duration, $queue->size(), $buffer->id );
			Settings::set_is_sending( false );
		} else {
			$processed_item_ids = $skipped_items_ids;
			$skipped_items_ids  = array();
		}

		if ( 'non-blocking' !== $processed_item_ids ) {
			if ( ! $processed_item_ids || is_wp_error( $processed_item_ids ) ) {
				$checked_in_item_ids = $queue->checkin( $buffer );
				if ( is_wp_error( $checked_in_item_ids ) ) {
					// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_error_log
					error_log( 'Error checking in buffer: ' . $checked_in_item_ids->get_error_message() );
					$queue->force_checkin();
				}
				if ( is_wp_error( $processed_item_ids ) ) {
					return new WP_Error( 'wpcom_error', $processed_item_ids->get_error_code() );
				}

				// Returning a wpcom_error is a sign to the caller that we should wait a while before syncing again.
				return new WP_Error( 'wpcom_error', 'jetpack_sync_send_data_false' );
			} else {
				// Detect if the last item ID was an error.
				$had_wp_error = is_wp_error( end( $processed_item_ids ) );
				if ( $had_wp_error ) {
					$wp_error = array_pop( $processed_item_ids );
				}
				// Also checkin any items that were skipped.
				if ( count( $skipped_items_ids ) > 0 ) {
					$processed_item_ids = array_merge( $processed_item_ids, $skipped_items_ids );
				}
				$processed_items = array_intersect_key( $items, array_flip( $processed_item_ids ) );
				/**
				 * Allows us to keep track of all the actions that have been sent.
				 * Allows us to calculate the progress of specific actions.
				 *
				 * @since 1.6.3
				 * @since-jetpack 4.2.0
				 *
				 * @param array $processed_actions The actions that we send successfully.
				 */
				do_action( 'jetpack_sync_processed_actions', $processed_items );
				$queue->close( $buffer, $processed_item_ids );
				// Returning a WP_Error is a sign to the caller that we should wait a while before syncing again.
				if ( $had_wp_error ) {
					return new WP_Error( 'wpcom_error', $wp_error->get_error_code() );
				}
			}
		}

		return true;
	}

	/**
	 * Immediately sends a single item without firing or enqueuing it
	 *
	 * @param string $action_name The action.
	 * @param array  $data The data associated with the action.
	 *
	 * @return Items processed. TODO: this doesn't make much sense anymore, it should probably be just a bool.
	 */
	public function send_action( $action_name, $data = null ) {
		if ( ! Settings::is_sender_enabled( 'full_sync' ) ) {
			return array();
		}

		// Compose the data to be sent.
		$action_to_send = $this->create_action_to_send( $action_name, $data );

		list( $items_to_send, $skipped_items_ids, $items, $preprocess_duration ) = $this->get_items_to_send( $action_to_send, true ); // phpcs:ignore VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedVariable
		Settings::set_is_sending( true );
		$processed_item_ids = apply_filters( 'jetpack_sync_send_data', $items_to_send, $this->get_codec()->name(), microtime( true ), 'immediate-send', 0, $preprocess_duration );
		Settings::set_is_sending( false );

		/**
		 * Allows us to keep track of all the actions that have been sent.
		 * Allows us to calculate the progress of specific actions.
		 *
		 * @param array $processed_actions The actions that we send successfully.
		 *
		 * @since 1.6.3
		 * @since-jetpack 4.2.0
		 */
		do_action( 'jetpack_sync_processed_actions', $action_to_send );

		return $processed_item_ids;
	}

	/**
	 * Create an synthetic action for direct sending to WPCOM during full sync (for example)
	 *
	 * @access private
	 *
	 * @param string $action_name The action.
	 * @param array  $data The data associated with the action.
	 * @return array An array of synthetic sync actions keyed by current microtime(true)
	 */
	private function create_action_to_send( $action_name, $data ) {
		return array(
			(string) microtime( true ) => array(
				$action_name,
				$data,
				get_current_user_id(),
				microtime( true ),
				Settings::is_importing(),
			),
		);
	}

	/**
	 * Returns any object that is able to be synced.
	 *
	 * @access public
	 *
	 * @param array $args the synchronized object parameters.
	 * @return string Encoded sync object.
	 */
	public function sync_object( $args ) {
		// For example: posts, post, 5.
		list( $module_name, $object_type, $id ) = $args;

		$sync_module = Modules::get_module( $module_name );
		$codec       = $this->get_codec();

		return $codec->encode( $sync_module->get_object_by_id( $object_type, $id ) );
	}

	/**
	 * Register additional sync XML-RPC methods available to Jetpack for authenticated users.
	 *
	 * @access public
	 * @since 1.6.3
	 * @since-jetpack 7.8.0
	 *
	 * @param array $jetpack_methods XML-RPC methods available to the Jetpack Server.
	 * @return array Filtered XML-RPC methods.
	 */
	public function register_jetpack_xmlrpc_methods( $jetpack_methods ) {
		$jetpack_methods['jetpack.syncObject'] = array( $this, 'sync_object' );
		return $jetpack_methods;
