ck whether block theme functions exist.
		if ( ! function_exists( 'get_block_template' ) ) {
			return null;
		}
		$active_theme     = \wp_get_theme()->get_stylesheet();
		$template_part_id = "{$active_theme}//{$template_part_name}";
		$template_part    = \get_block_template( $template_part_id, 'wp_template_part' );
		if ( is_wp_error( $template_part ) || empty( $template_part ) ) {
			return null;
		}
		return $template_part;
	}

	/**
	 * Returns true if  'header', 'footer' or 'home' has core search block
	 *
	 * @return boolean
	 */
	protected function template_parts_have_search_block() {
		$template_part_names = array( 'header', 'footer', 'home' );
		foreach ( $template_part_names as $part_name ) {
			$part = $this->get_template_part( $part_name );
			if ( $part instanceof \WP_Block_Template && static::content_has_search_block( $part->content ) ) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Append Search block to block if no 'wp:search' exists already.
	 *
	 * @param {string} $block_content - the content to append the search block.
	 */
	public static function inject_search_widget_to_block( $block_content ) {
		$search_block = sprintf(
			'<!-- wp:search {"label":"","buttonText":"%s"} /-->',
			__( 'Search', 'jetpack-search-pkg' )
		);

		// Place the search block on bottom of the first column if there's any.
		$column_end_pattern = '/(<\s*\/div[^>]*>\s*<!--\s*\/wp:column\s+[^>]*-->)/';
		if ( preg_match( $column_end_pattern, $block_content ) ) {
			return preg_replace( $column_end_pattern, "\n" . $search_block . "\n$1", $block_content, 1 );
		}

		// Place the search block on top of footer contents in the most inner group.
		$group_start_pattern = '/((<!--\s*wp:group\s[^>]*-->[.\s]*<\s*div[^>]*>\s*)+)/';
		if ( preg_match( $group_start_pattern, $block_content, $matches ) ) {
			return preg_replace( $group_start_pattern, "$1\n" . $search_block . "\n", $block_content, 1 );
		}

		return $block_content;
	}

	/**
	 * Autoconfig search by adding filter widgets
	 *
	 * @since  8.4.0
	 *
	 * @return array Array of config settings for search widget.
	 */
	protected function get_preconfig_widget_options() {
		$settings = array(
			'title'   => '',
			'filters' => array(),
		);

		$post_types = get_post_types(
			array(
				'public'   => true,
				'_builtin' => false,
			)
		);

		if ( ! empty( $post_types ) ) {
			$settings['filters'][] = array(
				'name'  => '',
				'type'  => 'post_type',
				'count' => 5,
			);
		}

		// Grab a maximum of 3 taxonomies.
		$taxonomies = array_slice(
			get_taxonomies(
				array(
					'public'   => true,
					'_builtin'