( is_readable( $customization_file ) ) {
			require_once $customization_file;
			$copy_dirs[] = $customization_file;
		}

		return $copy_dirs;
	}

	/**
	 * Determines whether jetpack_relatedposts is supported
	 *
	 * @return bool
	 */
	public function jetpack_relatedposts_supported() {
		$wpcom_related_posts_theme_blacklist = array(
			'Expound',
			'Traveler',
			'Opti',
			'Currents',
		);
		return ( ! in_array( wp_get_theme()->get( 'Name' ), $wpcom_related_posts_theme_blacklist, true ) );
	}

	/**
	 * Returns category details
	 *
	 * @param WP_Term $category Category object.
	 *
	 * @return array
	 */
	public function get_category_details( $category ) {
		return array(
			'value' => $category->term_id,
			'name'  => $category->name,
		);
	}

	/**
	 * Returns an option value as the result of the callable being applied to
	 * it if a value is set, otherwise null.
	 *
	 * @param string   $option_name   Option name.
	 * @param callable $cast_callable Callable to invoke on option value.
	 *
	 * @return int|null Numeric option value or null.
	 */
	protected function get_cast_option_value_or_null( $option_name, $cast_callable ) {
		$option_value = get_option( $option_name, null );
		if ( $option_value === null ) {
			return $option_value;
		}

		return call_user_func( $cast_callable, $option_value );
	}

	/**
	 * Collects the necessary information to return for a get settings response.
	 *
	 * @return array
	 */
	public function get_settings_response() {
		$response = array();

		// Allow update in later versions.
		/**
		 * Filter the structure of site settings to return.
		 *
		 * @module json-api
		 *
		 * @since 3.9.3
		 *
		 * @param array $site_format Data structure.
		 */
		$response_format = apply_filters( 'site_settings_site_format', self::$site_format );

		$blog_id = (int) $this->api->get_blog_id_for_output();
		/** This filter is documented in class.json-api-endpoints.php */
		$is_jetpack = true === apply_filters( 'is_jetpack_site', false, $blog_id );

		foreach ( array_keys( $response_format ) as $key ) {

			// refactoring to change lang parameter to locale in 1.2.
			$lang_or_locale = $this->get_locale( $key );
			if ( $lang_or_locale ) {
				$response[ $key ] = $lang_or_locale;
				continue;
			}

			switch ( $key ) {
				case 'ID':
					$response[ $key ] = $blog_id;
					break;
				case 'name':
					$response[ $key ] = (string) htmlspecialchars_decode( get_bloginfo( 'name' ), ENT_QUOTES );
					break;
				case 'description':
					$response[ $key ] = (string) htmlspecialchars_decode( get_bloginfo( 'description' ), ENT_QUOTES );
					break;
				case 'URL':
					$response[ $key ] = (string) home_url();
					break;
				case 'locale_variant':
					if ( function_exists( 'wpcom_l10n_get_blog_locale_variant' ) ) {
						$blog_locale_variant = wpcom_l10n_get_blog_locale_variant();
						if ( $blog_locale_variant ) {
							$response[ $key ] = $blog_locale_variant;
						}
					}
					break;
				case 'settings':
					$jetpack_relatedposts_options = Jetpack_Options::get_option( 'relatedposts', array() );
					// If the option's enabled key is NOT SET, it is considered enabled by the plugin.
					if ( ! isset( $jetpack_relatedposts_options['enabled'] ) ) {
						$jetpack_relatedposts_options['enabled'] = true;
					}

					if ( method_exists( 'Jetpack', 'is_module_active' ) ) {
						$jetpack_relatedposts_options['enabled'] = Jetpack::is_module_active( 'related-posts' );
					}

					$jetpack_search_supported = false;
					if ( function_exists( 'wpcom_is_jetpack_search_supported' ) ) {
						$jetpack_search_supported = wpcom_is_jetpack_search_supported( $blog_id );
					}

					$jetpack_search_active = false;
					if ( method_exists( 'Jetpack', 'is_module_active' ) ) {
						$jetpack_search_active = Jetpack::is_module_active( 'search' );
					}
					if ( function_exists( 'is_jetpack_module_active' ) ) {
						$jetpack_search_active = is_jetpack_module_active( 'search', $blog_id );
					}

					// array_values() is necessary to ensure the array starts at index 0.
					$post_categories = array_values(
						array_map(
							array( $this, 'get_category_details' ),
							get_categories( array( 'hide_empty' => false ) )
						)
					);

					$api_