 */
	protected function implode_array_field( $field ) {
		if ( is_array( $field ) ) {
			return implode( ' ', $field );
		}
		return $field;
	}

	/**
	 * Set the menu locations.
	 *
	 * @param array $menus - the menu list.
	 *
	 * @return array[]|WP_Error
	 */
	protected function set_locations( $menus ) {
		foreach ( $menus as $menu ) {
			if ( isset( $menu['locations'] ) ) {
				if ( true !== $this->locations_are_valid( $menu['locations'] ) ) {
					return $this->locations_are_valid( $menu['locations'] );
				}
			}
		}

		return array_map( array( $this, 'set_location' ), $menus );
	}

	/**
	 * Set the menu locations.
	 *
	 * @param array $menu - the menu.
	 *
	 * @return array
	 */
	protected function set_location( $menu ) {
		$this->set_menu_at_locations( $menu['locations'], $menu['id'] );
		return $menu;
	}

	/**
	 * Set the menu at locations.
	 *
	 * @param array $locations - the locations.
	 * @param int   $menu_id - the menu ID.
	 */
	protected function set_menu_at_locations( $locations, $menu_id ) {
		$location_map = get_nav_menu_locations();
		$this->remove_menu_from_all_locations( $menu_id, $location_map );

		if ( is_array( $locations ) ) {
			foreach ( $locations as $location ) {
				$location_map[ $location ] = $menu_id;
			}
		}

		set_theme_mod( 'nav_menu_locations', $location_map );

		$this->set_widget_menu_at_locations( $locations, $menu_id );
	}

	/**
	 * Remove from all locations.
	 *
	 * @param int   $menu_id - the menu ID.
	 * @param array $location_map - the location map.
	 */
	protected function remove_menu_from_all_locations( $menu_id, &$location_map ) {
		foreach ( get_nav_menu_locations() as $existing_location => $existing_menu_id ) {
			if ( $existing_menu_id === $menu_id ) {
				unset( $location_map[ $existing_location ] );
			}
		}
	}

	/**
	 * Set widget menu at locations.
	 *
	 * @param array $locations - the locations.
	 * @param int   $menu_id - the menu ID.
	 */
	protected function set_widget_menu_at_locations( $locations, $menu_id ) {
		$nav_menu_widgets = get_option( 'widget_nav_menu' );

		if ( ! is_array( $nav_menu_widgets ) ) {
			return;
		}

		// Remove menus from all custom menu widget locations
		foreach ( $nav_menu_widgets as &$widget ) {
			if ( is_array( $widget ) && isset( $widget['nav_menu'] ) && $widget['nav_menu'] === $menu_id