		$updated[ $key ] = false;
						}
					} elseif ( update_option( 'jetpack_api_cache_enabled', true ) ) {
						$updated[ $key ] = true;
					}
					break;

				case 'timezone_string':
					/*
					 * Map UTC+- timezones to gmt_offsets and set timezone_string to empty
					 * https://github.com/WordPress/WordPress/blob/4.4.2/wp-admin/options.php#L175
					 */
					if ( ! empty( $value ) && preg_match( '/^UTC[+-]/', $value ) ) {
						$gmt_offset = preg_replace( '/UTC\+?/', '', $value );
						if ( update_option( 'gmt_offset', $gmt_offset ) ) {
							$updated['gmt_offset'] = $gmt_offset;
						}

						$value = '';
					}

					/*
					 * Always set timezone_string either with the given value or with an
					 * empty string
					 */
					if ( update_option( $key, $value ) ) {
						$updated[ $key ] = $value;
					}
					break;

				case 'woocommerce_onboarding_profile':
					// Allow boolean values but sanitize_text_field everything else.
					$sanitized_value = (array) $value;
					array_walk_recursive(
						$sanitized_value,
						function ( &$value ) {
							if ( ! is_bool( $value ) ) {
								$value = sanitize_text_field( $value );
							}
						}
					);
					if ( update_option( $key, $sanitized_value ) ) {
						$updated[ $key ] = $sanitized_value;
					}
					break;

				case 'woocommerce_store_address':
				case 'woocommerce_store_address_2':
				case 'woocommerce_store_city':
				case 'woocommerce_default_country':
				case 'woocommerce_store_postcode':
					$sanitized_value = sanitize_text_field( $value );
					if ( update_option( $key, $sanitized_value ) ) {
						$updated[ $key ] = $sanitized_value;
					}
					break;

				case 'date_format':
				case 'time_format':
					// settings are stored as strings.
					// raw_value is used to help preserve any escaped characters that might exist in the formatted string.
					$sanitized_value = sanitize_text_field( $raw_value );
					if ( update_option( $key, $sanitized_value ) ) {
						$updated[ $key ] = $sanitized_value;
					}
					break;

				case 'start_of_week':
					// setting is stored as int in 0-6 range (days of week).
					$coerce_value = (int) $value;
					$limit_value  = ( $coerce_value >= 0 && $coerce_value <= 6 ) ? $coerce_value : 0;
					if ( update_option( $key, $limit_value ) ) {
						$updated[ $key ] = $limit_value;
					}
					break;

				case 'site_icon':
					/*
					 * settings are stored as deletable numeric (all empty
					 * values as delete intent), validated as media image
					 */
					if ( empty( $value ) || WPCOM_JSON_API::is_falsy( $value ) ) {
						/**
						 * Fallback mechanism to clear a third party site icon setting. Can be used
						 * to unset the option when an API request instructs the site to remove the site icon.
						 *
						 * @module json-api
						 *
						 * @since 4.10
						 */
						if ( delete_option( $key ) || apply_filters( 'rest_api_site_icon_cleared', false ) ) {
							$updated[ $key ] = null;
						}
					} elseif ( is_numeric( $value ) ) {
						$coerce_value = (int) $value;
						if ( wp_attachment_is_image( $coerce_value ) && update_option( $key, $coerce_value ) ) {
							$updated[ $key ] = $coerce_value;
						}
					}
					break;

				case Jetpack_SEO_Utils::FRONT_PAGE_META_OPTION:
					if ( ! Jetpack_SEO_Utils::is_enabled_jetpack_seo() && ! Jetpack_SEO_Utils::has_legacy_front_page_meta() ) {
						return new WP_Error( 'unauthorized', __( 'SEO tools are not enabled for this site.', 'jetpack' ), 403 );
					}

					if ( ! is_string( $value ) ) {
						return new WP_Error( 'invalid_input', __( 'Inva