adata keys and values. All unprotected meta keys are available by default for read requests. Both unprotected and protected meta keys are available for authenticated requests with access. Protected meta keys can be made available with the <code>rest_api_allowed_public_metadata</code> filter.',
		'meta'             => '(object) API result meta data',
		'capabilities'     => '(object) List of post-specific permissions for the user; publish_post, edit_post, delete_post',
		'revisions'        => '(array) List of post revision IDs. Only available for posts retrieved with context=edit.',
		'other_URLs'       => '(object) List of URLs for this post. Permalink and slug suggestions.',
	);

	/**
	 * Constructor function.
	 *
	 * @param string|array|object $args — Args.
	 */
	public function __construct( $args ) {
		if ( is_array( $this->post_object_format ) && isset( $this->post_object_format['format'] ) ) {
			$this->post_object_format['format'] = get_post_format_strings();
		}
		if ( ! $this->response_format ) {
			$this->response_format =& $this->post_object_format;
		}
		parent::__construct( $args );
	}

	/**
	 * Get a post by a specified field and value
	 *
	 * @param string $field - the field.
	 * @param string $field_value - the field value.
	 * @param string $context Post use context (e.g. 'display').
	 *
	 * @return array|WP_Error Post
	 **/
	public function get_post_by( $field, $field_value, $context = 'display' ) {

		// validate input
		if ( ! in_array( $field, array( 'ID', 'name' ), true ) ) {
			return new WP_Error( 'invalid_field', 'Invalid API FIELD', 400 );
		}

		if ( ! in_array( $context, array( 'display', 'edit' ), true ) ) {
			return new WP_Error( 'invalid_context', 'Invalid API CONTEXT', 400 );
		}

		if ( 'display' === $context ) {
			$args = $this->query_args();
			if ( isset( $args['content_width'] ) && $args['content_width'] ) {
				$GLOBALS['content_width'] = (int) $args['content_width'];
			}
		}

		if ( isset( $_SERVER['HTTP_USER_AGENT'] ) && strpos( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ), 'wp-windows8' ) ) { // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized -- we're not using this value and making changes, just checking if it e