seif ( is_array( $locations ) ) {
			foreach ( $locations as $location_name ) {
				if ( ! $this->location_name_exists( $location_name ) ) {
					return new WP_Error(
						'locations-array',
						sprintf( "Location '%s' does not exist.", $location_name ),
						404
					);
				}
			}
			return true;
		}
		return new WP_Error( 'locations', 'Locations must be array or integer.', 400 );
	}

	/**
	 * Check if the location name exists.
	 *
	 * @param string $location_name - the location name.
	 *
	 * @return bool
	 */
	protected function location_name_exists( $location_name ) {
		$widget_location_names = wp_list_pluck( WPCOM_JSON_API_Menus_Widgets::get(), 'name' );

		$existing_locations = get_nav_menu_locations();

		if ( ! is_array( get_registered_nav_menus() ) ) {
			return false;
		}

		return array_key_exists( $location_name, get_registered_nav_menus() ) ||
			array_key_exists( $location_name, $existing_locations ) ||
			in_array( $location_name, $widget_location_names, true );
	}

}

new WPCOM_JSON_API_Menus_New_Menu_Endpoint(
	array(
		'method'               => 'POST',
		'description'          => 'Create a new navigation menu.',
		'group'                => 'menus',
		'stat'                 => 'menus:new-menu',
		'path'                 => '/sites/%s/menus/new',
		'path_labels'          => array(
			'$site' => '(int|string) Site ID or domain',
		),
		'request_format'       => array(
			'name' => '(string) Name of menu',
		),
		'response_format'      => array(
			'id' => '(int) Newly created menu ID',
		),
		'example_request'      => 'https://public-api.wordpress.com/rest/v1.1/sites/82974409/menus/new',
		'example_request_data' => array(
			'headers' => array( 'authorization' => 'Bearer YOUR_API_TOKEN' ),
			'body'    => array(
				'name' => 'Menu 1',
			),
		),
	)
);

/**
 * New menu endpoint class.
 */
class WPCOM_JSON_API_Menus_New_Menu_Endpoint extends WPCOM_JSON_API_Menus_Abstract_Endpoint {

	/**
	 * The API Callback.
	 *
	 * @param string $path - the path.
	 * @param int    $site - the site ID.
	 *
	 * @return array|WP_Error
	 */
	public function callback( $path = '', $site = 0 ) {
		$site_id = $this->switch_to_blog_and_validate_user( $this->api->get_blog_id( $site ) );

		if ( is_wp_error( $site_id ) ) {
			return $site_id;
		}

		$data = $this->input();

		$id = wp_create_nav_menu( $data['name'] );

		if ( is_wp_error( $id ) ) {
			return $id;
		}

		return array( 'id' => $id );
	}
}

new WPCOM_JSON_API_Menus_Update_Menu_Endpoint(
	array(
		'method'               => 'POST',
		'description'          => 'Update a navigation menu.',
		'group'                => 'menus',
		'stat'                 => 'menus:update-menu',
		'path'                 => '/sites/%s/menus/%d',
		'path_labels'          => array(
			'$site'    => '(int|string) Site ID or domain',
			'$menu_id' => '(int) Menu ID',
		),
		'request_format'       => array(
			'name'  => '(string) Name of menu',
			'items' => '(array) A list of menu item objects.
			<br/><br/>
			Item objects contain fields relating to that item, e.g. id, type, content_id,
			but they can also contain other items objects - this nesting represents parents
			and child items in the item tree.',
		),
		'response_format'      => array(
			'menu' => '(object) Updated menu object',
		),
		'example_request'      => 'https://public-api.wordpress.com/rest/v1.1/sites/82974409/menus/510604099',
		'example_request_data' => array(
			'headers' => array( 'authorization' => 'Bearer YOUR_API_TOKEN' ),
			'body'    => array(
				'name' => 'Test Menu',
			),
		),
	)
);

/**
 * Update menu endpoint class.
 */
class WPCOM_JSON_API_Menus_Update_Menu_Endpoint extends WPCOM_JSON_API_Menus_Abstract_Endpoint {

	/**
	 * The API Callback.
	 *
	 * @param string $path - the path.
	 * @param int    $site - the site ID.
	 * @param int    $menu_id - the menu ID.
	 *
	 * @return array|WP_Error
	 */
	public function callback( $path = '', $site = 0, $menu_id = 0 ) {
		$site_id = $this->switch_to_blog_and_validate_user( $this->api->get_blog_id( $site ) );

		if ( is