) ) ) {
			return new WP_Error( 'missing_jetpack_module', 'The Sharing module must be activated in order to use this endpoint', 400 );
		}
	}

	/**
	 * Format the sharing button.
	 *
	 * @param object $button - the button object.
	 * @return array
	 */
	public function format_sharing_button( $button ) {
		$response = array(
			'ID'        => $button->get_id(),
			'name'      => $button->get_name(),
			'shortname' => $button->shortname,
			'custom'    => is_a( $button, 'Share_Custom' ),
			'enabled'   => $this->is_button_enabled( $button ),
		);

		if ( $response['enabled'] ) {
			// Status is either "disabled" or the visibility value
			$response['visibility'] = $this->get_button_visibility( $button );
		}

		if ( ! empty( $button->icon ) ) {
			// Only pre-defined sharing buttons include genericon
			$response['genericon'] = $button->icon;
		}

		if ( method_exists( $button, 'get_options' ) ) {
			// merge get_options() values into response, primarily to account
			// for custom sharing button values
			foreach ( $button->get_options() as $key => $value ) {
				// Capitalize URL property
				if ( 'url' === strtolower( $key ) ) {
					$key = strtoupper( $key );
				}

				$response[ $key ] = $value;
			}
		}

		return $response;
	}

	/**
	 * Get the button visibility.
	 *
	 * @param object $button - the button object.
	 *
	 * @return string|false
	 */
	public function get_button_visibility( $button ) {
		$services     = $this->sharing_service->get_blog_services();
		$visibilities = self::$all_visibilities;
		$button_id    = $button->get_id();

		foreach ( $visibilities as $visibility ) {
			if ( isset( $services[ $visibility ][ $button_id ] ) ) {
				return $visibility;
			}
		}

		return false;
	}

	/**
	 * Check if the button is enabled.
	 *
	 * @param object $button - the button object.
	 *
	 * @return bool
	 */
	public function is_button_enabled( $button ) {
		return false !== $this->get_button_visibility( $button );
	}

	/**
	 * Check if button is for custom (?).
	 *
	 * @param array $button - the button array.
	 *
	 * @return bool
	 */
	protected function is_button_input_for_custom( $button ) {
		return ( isset( $button['custom'] ) && $button['custom'] ) ||
			( isset( $button['ID'] ) && 1 === preg_match( '/^custom-/', $button['ID'] ) ) ||
			! empty( $button['name'] ) || ! empty( $button['URL'] ) || ! empty( $button['icon'] );
	}

	/**
	 * Validate the button input.
	 *
	 * @param array $button - the button array.
	 * @param bool  $is_new - if the button is new.
	 *
	 * @return null|WP_Error
	 */
	protected function validate_button_input( $button, $is_new = false ) {
		if ( ! empty( $button['visibility'] ) && ! in_array( $button['visibility'], self::$all_visibilities, true ) ) {
			return new WP_Error( 'invalid_visibility', sprintf( 'The visibility field must be one of the following values: %s', implode( ', ', self::$all_visibilities ) ), 400 );
		} elseif ( $is_new && empty( $button['URL'] ) ) {
			return new WP_Error( 'invalid_request', 'The URL field is required', 400 );
		} elseif ( $is_new && empty( $button['icon'] ) ) {
			return new WP_Error( 'invalid_request', 'The icon field is required', 400 );
		}
	}

	/**
	 * Create a custom button.
	 *
	 * @param array $button - the button array.
	 *
	 * @return Share_Custom|false
	 */
	public function create_custom_button( $button ) {
		// Default visibility to 'visible' if enabled
		if ( empty( $button['visibility'] ) && true === $button['enabled'] ) {
			$button['visibility'] = 'visible';
		}

		$updated_service = $this->sharing_service->new_service( $button['name'], $button['URL'], $button['icon'] );
		if ( false !== $updated_service && ( true === $button['enabled'] || ! empty( $button['visibility'] ) ) ) {
			$blog_services = $this->sharing_service->get_blog_services();
			$blog_services[ $button['visibility'] ][ (string) $updated_service->get_id() ] = $updated_service;
			$this->sharing_service->set_blog_services( array_keys( $blog_services['visible'] ), array_keys( $blog_services['hidden'] ) );
		}

		return $updated_service;
	}

	/**
	 * Update the button.
	 *
	 * @param int   $button_id - the button id.
	 * @param array $button - the button array.
	 *
	 * @return Share_Custom|WP_Error
	 */
	public function update_button( $button_id, $button ) {
		$blog_services = $this->sharing_service->get_blog_services();

		// Find existing button
		$all_buttons = $this->sharing_service->get_all_services_blog();
		if ( ! array_key_exists( $button_id, $all_buttons ) ) {
			// Button doesn't exi