alt'                        => '(string)  Alternative text for image files.',
			'thumbnails'                 => '(object) Media item thumbnail URL options',
			'height'                     => '(int) (Image & video only) Height of the media item',
			'width'                      => '(int) (Image & video only) Width of the media item',
			'length'                     => '(int) (Video & audio only) Duration of the media item, in seconds',
			'exif'                       => '(array) (Image & audio only) Exif (meta) information about the media item',
			'videopress_guid'            => '(string) (Video only) VideoPress GUID of the video when uploaded on a blog with VideoPress',
			'videopress_processing_done' => '(bool) (Video only) If the video is uploaded on a blog with VideoPress, this will return the status of processing on the video.',
			'revision_history'           => '(object) An object with `items` and `original` keys. ' .
									'`original` is an object with data about the original image. ' .
									'`items` is an array of snapshots of the previous images of this Media. ' .
									'Each item has the `URL`, `file, `extension`, `date`, and `mime_type` fields.',
		),

		'example_request'      => 'https://public-api.wordpress.com/rest/v1.2/sites/82974409/media/446',
		'example_request_data' => array(
			'headers' => array(
				'authorization' => 'Bearer YOUR_API_TOKEN',
			),
			'body'    => array(
				'title' => 'Updated Title',
			),
		),
	)
);

/**
 * Edit media v1_2 endpoint class.
 */
class WPCOM_JSON_API_Edit_Media_v1_2_Endpoint extends WPCOM_JSON_API_Update_Media_v1_1_Endpoint { //phpcs:ignore
	/**
	 * Return an array of mime_type items allowed when the media file is uploaded.
	 *
	 * @param Array $default_mime_types - array of default mime types.
	 *
	 * @return {Array} mime_type array
	 */
	public static function get_allowed_mime_types( $default_mime_types ) {
		return array_unique(
			array_merge(
				$default_mime_types,
				array(
					'application/msword',                                                         // .doc
					'application/vnd.ms-powerpoint',                                              // .ppt, .pps
					'application/vnd.ms-excel',                                                   // .xls
					'application/vnd.openxmlformats-officedocument.presentationml.presentation',  // .pptx
					'application/vnd.openxmlformats-officedocument.presentationml.slideshow',     // .ppsx
					'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',          // .xlsx
					'application/vnd.openxmlformats-officedocument.wordprocessingml.document',    // .docx
					'application/vnd.oasis.opendocument.text',                                    // .odt
					'application/pdf',                                                            // .pdf
				)
			)
		);
	}

	/**
	 * Update the media post grabbing the post values from
	 * the `attrs` parameter
	 *
	 * @param 