etpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'response_format'         => $sync_settings_response,
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/settings',
	)
);

// POST /sites/%s/sync/settings
new Jetpack_JSON_API_Sync_Modify_Settings_Endpoint(
	array(
		'description'             => 'Update sync settings',
		'method'                  => 'POST',
		'group'                   => '__do_not_document',
		'path'                    => '/sites/%s/sync/settings',
		'stat'                    => 'write-sync-settings',
		'allow_jetpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'request_format'          => $sync_settings_response,
		'response_format'         => $sync_settings_response,
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/settings',
	)
);

// GET /sites/%s/sync/object
new Jetpack_JSON_API_Sync_Object(
	array(
		'description'             => 'Get an object by ID from one of the sync modules, in the format it would be synced in',
		'group'                   => '__do_not_document',
		'method'                  => 'GET',
		'path'                    => '/sites/%s/sync/object',
		'stat'                    => 'sync-object',
		'allow_jetpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'query_parameters'        => array(
			'module_name' => '(string) The sync module ID, e.g. "posts"',
			'object_type' => '(string) An identified for the object type, e.g. "post"',
			'object_ids'  => '(array) The IDs of the objects',
		),
		'response_format'         => array(
			'objects' => '(string) The encoded objects',
			'codec'   => '(string) The codec used to encode the objects, deflate-json-array or simple',
		),
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/object?module_name=posts&object_type=post&object_ids[]=1&object_ids[]=2&object_ids[]=3',
	)
);

// POST /sites/%s/sync/now
new Jetpack_JSON_API_Sync_Now_Endpoint(
	array(
		'description'             => 'Force immediate sync of top items on a queue',
		'method'                  => 'POST',
		'path'                    => '/sites/%s/sync/now',
		'stat'                    => 'sync-now',
		'allow_jetpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'request_format'          => array(
			'queue' => '(string) sync or full_sync',
		),
		'response_format'         => array(
			'response' => '(array) The response from the server',
		),
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/now?queue=full_sync',
	)
);

// POST /sites/%s/sync/unlock
new Jetpack_JSON_API_Sync_Unlock_Endpoint(
	array(
		'description'             => 'Unlock the queue in case it gets locked by a process.',
		'method'                  => 'POST',
		'path'                    => '/sites/%s/sync/unlock',
		'group'                   => '__do_not_document',
		'stat'                    => 'sync-unlock',
		'allow_jetpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'request_format'          => array(
			'queue' => '(string) sync or full_sync',
		),
		'response_format'         => array(
			'success' => '(bool) Unlocking the queue successful?',
		),
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/unlock',
	)
);

// GET /sites/%s/sync/object-id-range
new Jetpack_JSON_API_Sync_Object_Id_Range(
	array(
		'description'             => 'Gets minimum and maximum object ids for each batch of given batch size.',
		'method'                  => 'GET',
		'path'                    => '/sites/%s/sync/object-id-range',
		'group'                   => '__do_not_document',
		'stat'                    => 'sync-object-id-range',
		'allow_jetpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'query_parameters'        => array(
			'batch_size'  => '(int=1000) The amount of objects per batch.',
			'sync_module' => '(string=posts) The sync module used to enumerate the ranges.',
		),
		'response_format'         => array(
			'ranges' => '(array) An array of range objects with min and max properties for each batch.',
		),
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/object-id-range?batch_size=100&sync_module=comments',
	)
);

// POST /sites/%s/sync/checkout
new Jetpack_JSON_API_Sync_Checkout_Endpoint(
	array(
		'description'             => 'Locks the queue and returns items and the buffer ID.',
		'method'                  => 'POST',
		'path'                    => '/sites/%s/sync/checkout',
		'group'                   => '__do_not_document',
		'stat'                    => 'sync-checkout',
		'allow_jetpack_site_auth' => true,
		'path_labels'             => array(
			'$site' => '(int|string) The site ID, The site domain',
		),
		'request_format'          => array(
			'queue'           => '(string) sync or full_sync',
			'number_of_items' => '(int=10) Maximum number of items from the queue to be returned',
			'encode'          => '(bool=true) Use the default encode method',
			'force'           => '(bool=false) Force unlock the queue',
			'pop'             => '(bool=false) Pop from the queue without checkout, use carefully 😱',
		),
		'response_format'         => array(
			'buffer_id'      => '(string) Buffer ID that we are using',
			'items'          => '(array) Items from the queue that are ready to be processed by the sync server',
			'skipped_items'  => '(array) Skipped item ids',
			'codec'          => '(string) The name of the codec used to encode the data',
			'sent_timestamp' => '(int) Current timestamp of the server',
		),
		'example_request'         => 'https://public-api.wordpress.com/rest/v1.1/sites/example.wordpress.org/sync/checkout',
	)
);

// POST /sites/%s/sync/close
new Jetpack_JSON_API_Sync_Close_Endpoint(
	array(
		'description'             => 'Closes the buffer and delete the processed items from the q